#ifndef TT_COMMAND_LINE_ANALYZER_H_
#define TT_COMMAND_LINE_ANALYZER_H_

#include <string>
#include <vector>
#include <map>

// -- TtCommandLineOption ------------------------------------------------
class TtCommandLineOption {
public:
  explicit TtCommandLineOption( char option, bool is_indispensable = false );
  virtual ~TtCommandLineOption();

  char GetOption( void ) const;

  void SetIndispensable( bool flag );
  bool IsIndispensable( void ) const;

  virtual void Analyze( const std::vector<std::string> operand ) = 0;

private:
  char option_;
  bool is_indispensable_;
};

// -- TtCommandLineOptionSingleValue -------------------------------------
template <class TYPE>
class TtCommandLineOptionSingleValue : public TtCommandLineOption {
public:
  explicit TtCommandLineOptionSingleValue( char option, TYPE* value, bool is_indispensable = false ) :
  TtCommandLineOption( option, is_indispensable ), value_( value ) {}

  void SetDefaultValue( TYPE default_value ) {
    default_value_ = default_value;
  }
  TYPE GetDefaultValue( void ) const {
    return default_value_;
  }

protected:
  TYPE* value_;
  TYPE  default_value_;
};

// -- TtCommandLineOptionBoolean -----------------------------------------
class TtCommandLineOptionBoolean : public TtCommandLineOptionSingleValue<bool> {
public:
  explicit TtCommandLineOptionBoolean( char option, bool* value );

  virtual void Analyze( const std::vector<std::string> operand );
};

// -- TtCommandLineOptionInteger -----------------------------------------
class TtCommandLineOptionInteger : public TtCommandLineOptionSingleValue<int> {
public:
  explicit TtCommandLineOptionInteger( char option, int* value, bool is_indispensable = false );

  virtual void Analyze( const std::vector<std::string> operand );
};

// -- TtCommandLineOptionString ------------------------------------------
class TtCommandLineOptionString : public TtCommandLineOptionSingleValue<std::string> {
public:
  explicit TtCommandLineOptionString( char option, std::string* value, bool is_indispensable = false );

  virtual void Analyze( const std::vector<std::string> operand );
};


// -- TtCommandLineOptionAnalyzer ----------------------------------------------
class TtCommandLineOptionAnalyzer {
public:
  explicit TtCommandLineOptionAnalyzer( std::vector<std::string>& rest );
  ~TtCommandLineOptionAnalyzer();

  void Regist( TtCommandLineOption& option );

  void AnalyzeForConsole( unsigned int argc, const char** argv );
  void AnalyzeForWindows( const std::string& args_string );

private:
  void Analyze( const std::vector<std::string> args );

private:
  std::vector<std::string>&            rest_;
  std::map<char, TtCommandLineOption*> table_;
};

#endif // TT_COMMAND_LINE_ANALYZER_H_
