#include "ttl_define.h"
#include "tt_scroll.h"

DWORD
TtScroll::GetStyle( void )
{
  return 0;
}

DWORD
TtScroll::GetExtendedStyle( void )
{
  return WS_EX_CLIENTEDGE;
}

PCTSTR
TtScroll::GetClassName( void )
{
  return "SCROLLBAR";
}


int
TtScroll::GetTrackPosition( void )
{
  this->GetInfo();
  return info_.nPos;
}

void
TtScroll::SetTrackPosition( int position )
{
  this->GetInfo();
  info_.nPos = position;
  this->SetInfo();
}

int
TtScroll::GetMinimum( void )
{
  this->GetInfo();
  return info_.nMin;
}

void
TtScroll::SetMinimum( int min )
{
  this->GetInfo();
  info_.nMin = min;
  this->SetInfo();
}

int
TtScroll::GetMaximum( void )
{
  this->GetInfo();
  return info_.nMax;
}

void
TtScroll::SetMaximum( int max )
{
  this->GetInfo();
  info_.nMax = max;
  this->SetInfo();
}

void
TtScroll::SetRange( int min, int max )
{
  this->GetInfo();
  info_.nMin = min;
  info_.nMax = max;
  this->SetInfo();
}

int
TtScroll::GetPageSize( void )
{
  this->GetInfo();
  return info_.nPage;
}

void
TtScroll::SetPageSize( int page_size )
{
  this->GetInfo();
  info_.nPage = page_size;
  this->SetInfo();
}

bool TtScroll::GetInfo( void )
{
  info_.fMask = SIF_ALL;
  return GetScrollInfo( this->GetHandle(), SB_CTL, &info_ );
}

void TtScroll::SetInfo( void )
{
  info_.fMask |= SIF_DISABLENOSCROLL;
  SetScrollInfo( this->GetHandle(), SB_CTL, &info_, TRUE );
}

LRESULT
TtScroll::Scrolling( int code, int position )
{
  this->GetInfo();
  switch ( code ) {
  case SB_LEFT:
    info_.nPos = info_.nMin;
    break;

  case SB_RIGHT:
    info_.nPos = info_.nMax;
    break;

  case SB_LINELEFT:
    if ( info_.nPos > info_.nMin ) {
      info_.nPos -= 1;
    }
    break;

  case SB_LINERIGHT:
    if ( info_.nPos < info_.nMax - 1 ) {
      info_.nPos += 1;
    }
    break;

  case SB_PAGELEFT:
    info_.nPos -= info_.nPage;
    if ( info_.nPos < info_.nMin ) {
      info_.nPos = info_.nMin;
    }
    break;

  case SB_PAGERIGHT:
    info_.nPos += info_.nPage;
    if ( info_.nPos > info_.nMax ) {
      info_.nPos = info_.nMax;
    }
    break;

  case SB_THUMBPOSITION:
    info_.nPos = position;
    break;

  default:
    return TtWindow::MSG_DEFAULT;
  }
  this->SetInfo();
  return 0;
}

// -- TtScrollVertical ---------------------------------------------------
DWORD
TtScrollVertical::GetStyle( void )
{
  return SBS_VERT;
}

LRESULT
TtScrollVertical::WMVerticalScrollReceived( int code, int position, HWND handle )
{
  NOT_USE_VAR( handle );
  return this->Scrolling( code, position );
}

// -- TtScrollVertical ---------------------------------------------------
DWORD
TtScrollHorizontal::GetStyle( void )
{
  return SBS_HORZ;
}

LRESULT
TtScrollHorizontal::WMHorizontalScrollReceived( int code, int position, HWND handle )
{
  NOT_USE_VAR( handle );
  return this->Scrolling( code, position );
}

// -- TtTrack ------------------------------------------------------------
DWORD
TtTrack::GetStyle( void )
{
  return TBS_AUTOTICKS;
}

DWORD
TtTrack::GetExtendedStyle( void )
{
  return 0;
}

PCTSTR
TtTrack::GetClassName( void )
{
  return TRACKBAR_CLASS;
}


int
TtTrack::GetTrackPosition( void )
{
  return this->SendMessage( TBM_GETPOS );
}

void
TtTrack::SetTrackPosition( int position )
{
  this->SendMessage( TBM_SETPOS, TRUE, position );
}

int
TtTrack::GetMinimum( void )
{
  return this->SendMessage( TBM_GETRANGEMIN );
}

void
TtTrack::SetMinimum( int min )
{
  this->SendMessage( TBM_SETRANGEMIN, TRUE, min );
}

int
TtTrack::GetMaximum( void )
{
  return this->SendMessage( TBM_GETRANGEMAX );
}

void
TtTrack::SetMaximum( int max )
{
  this->SendMessage( TBM_SETRANGEMAX, TRUE, max );
}

void
TtTrack::SetRange( int min, int max )
{
  this->SetMinimum( min );
  this->SetMaximum( max );
}

int
TtTrack::GetPageSize( void )
{
  return this->SendMessage( TBM_GETPAGESIZE );
}

void
TtTrack::SetPageSize( int page_size )
{
  this->SendMessage( TBM_SETPAGESIZE, 0, page_size );
}

int
TtTrack::GetLineSize( void )
{
  return this->SendMessage( TBM_GETLINESIZE );
}

void
TtTrack::SetLineSize( int line_size )
{
  this->SendMessage( TBM_SETLINESIZE, 0, line_size );
}

void
TtTrack::SetTickFrequency( unsigned int frequency )
{
  this->SendMessage( TBM_SETTICFREQ, frequency );
}

void
TtTrack::ClearTick( void )
{
  this->SendMessage( TBM_CLEARTICS, TRUE );
}


LRESULT
WMVerticalScrollReceived( int code, int position, HWND handle )
{
  NOT_USE_VAR( code );
  NOT_USE_VAR( position );
  NOT_USE_VAR( handle );
  return TtWindow::MSG_DEFAULT;
}

LRESULT
WMHorizontalScrollReceived( int code, int position, HWND handle )
{
  NOT_USE_VAR( code );
  NOT_USE_VAR( position );
  NOT_USE_VAR( handle );
  return TtWindow::MSG_DEFAULT;
}


// -- TtTrackHorizontal --------------------------------------------------
DWORD
TtTrackHorizontal::GetStyle( void )
{
  return TBS_AUTOTICKS | TBS_HORZ;
}

// -- TtTrackVertical ----------------------------------------------------
DWORD
TtTrackVertical::GetStyle( void )
{
  return TBS_AUTOTICKS | TBS_VERT;
}
