#include "ttl_define.h"
#include "tt_exception.h"
#include "tt_tab.h"

DWORD
TtTab::GetStyle( void )
{
  return WS_TABSTOP;
}

DWORD
TtTab::GetExtendedStyle( void )
{
  return 0;
}

PCTSTR
TtTab::GetClassName( void )
{
  return WC_TABCONTROL;
}


int
TtTab::GetCount( void )
{
  return this->SendMessage( TCM_GETITEMCOUNT );
}

int
TtTab::InsertAt( int index, const std::string& str )
{
  TCITEM tc_item;
  tc_item.mask = TCIF_TEXT;
  tc_item.pszText = const_cast<char*>( str.c_str() );
  int ret = TabCtrl_InsertItem( this->GetHandle(), index, &tc_item );
  if ( ret == -1 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( REAL_FUNC_NAME_OF( TabCtrl_InsertItem ) );
  }
  if ( index + 1 > static_cast<int>( control_set_.size() ) ) {
    control_set_.resize( index + 1 );
  }
  return ret;
}

int
TtTab::DeleteAt( int index )
{
  int ret = this->SendMessage( TCM_DELETEITEM, index );
  control_set_.erase( control_set_.begin() + index );
  return ret;
}

int
TtTab::Clear( void )
{
  return this->SendMessage( TCM_DELETEALLITEMS );
}

int
TtTab::SetCurrent( int index )
{
  return this->SendMessage( TCM_SETCURSEL, index );
}

int
TtTab::GetCurrent( void )
{
  return TabCtrl_GetCurSel( this->GetHandle() );
}

RECT
TtTab::GetViewRect( void )
{
  RECT rect = this->GetRectangle();;
  this->SendMessage( TCM_ADJUSTRECT, 0, reinterpret_cast<LPARAM>( &rect ) );
  return rect;
}


void
TtTab::RegistControl( unsigned int index, TtWindow& control )
{
  this->RegistControl( index, control, true );
}

void
TtTab::RegistControl( unsigned int index, TtWindow& control, bool visible )
{
  control_set_[index].insert( &control );
  control_last_state_[&control] = visible;
  if ( static_cast<int>( index ) != this->GetCurrent() ) {
    control.Hide();
  }
  else if ( visible ) {
    control.Show();
  }
}

void
TtTab::SaveCurrentVisibleState( void )
{
  for ( std::set<TtWindow*>::iterator it = control_set_[this->GetCurrent()].begin(); 
        it != control_set_[this->GetCurrent()].end(); ++it ) {
    control_last_state_[*it] = (*it)->GetVisible();
  }
}

void
TtTab::HideCurrentRegistedControls( void )
{
  for ( std::set<TtWindow*>::iterator it = control_set_[this->GetCurrent()].begin(); 
        it != control_set_[this->GetCurrent()].end(); ++it ) {
    (*it)->Hide();
  }
}


void
TtTab::LoadCurrentVisibleState( void )
{
  for ( std::set<TtWindow*>::iterator it = control_set_[this->GetCurrent()].begin(); 
        it != control_set_[this->GetCurrent()].end(); ++it ) {
    if ( control_last_state_[*it] ) {
      (*it)->Show();
    }
  }
}
