#include <process.h>
#include <errno.h>

#include <stdexcept>

#include "ttl_define.h"
#include "tt_thread.h"
#include "tt_exception.h"


unsigned int __stdcall
TtThread::EntryPoint( void* arg )
{
  _endthreadex( reinterpret_cast<TtThread*>( arg )->Run() );
  // dummy
  return 0;
}


unsigned int
TtThread::GetCurrentThreadId( void )
{
  return ::GetCurrentThreadId();
}


TtThread::TtThread( void ) :
thread_handle_( INVALID_HANDLE_VALUE ),
thread_id_( 0 )
{
}


TtThread::~TtThread()
{
  if ( thread_handle_ != INVALID_HANDLE_VALUE ) {
    CloseHandle( thread_handle_ );
  }
}


void
TtThread::Start( void )
{
  if ( thread_handle_ != INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  unsigned int tmp_id;
  long int tmp_handle = _beginthreadex( NULL, 0, EntryPoint, this, 0, &tmp_id );
  if ( tmp_handle == -1L ) {
    throw TT_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( _beginthreadex ), errno );
  }
  thread_handle_ = reinterpret_cast<HANDLE>( tmp_handle );
  thread_id_ = static_cast<DWORD>( tmp_id );
}


void
TtThread::Join( void )
{
  if ( thread_handle_ == INVALID_HANDLE_VALUE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  if ( WaitForSingleObject( thread_handle_, INFINITE ) != WAIT_OBJECT_0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( WaitForSingleObject ) );
  }
}


void
TtThread::Terminate( DWORD exit_code )
{
  if ( TerminateThread( thread_handle_, exit_code ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( TerminateThread ) );
  }
}


DWORD
TtThread::GetExitCode( void ) const
{
  DWORD ret;
  if ( GetExitCodeThread( thread_handle_, &ret ) == 0 ) {
    throw TT_WIN_SYSTEM_CALL_EXCEPTION( FUNC_NAME_OF( GetExitCodeThread ) );
  }
  if ( ret == STILL_ACTIVE ) {
    throw TtInvalidOperationException( typeid( *this ) );
  }
  return ret;
}


DWORD
TtThread::GetThreadId( void ) const
{
  return thread_id_;
}
