#ifndef TT_WINDOW_H_
#define TT_WINDOW_H_
#define OEMRESOURCE

#include <windows.h>
#include <commctrl.h>

#include <string>
#include <map>
#include <vector>

#include "tt_image_list.h"
#include "tt_font.h"

#define TT_WINDOW_MSG_DEFAULT    0x8000
#define TT_WINDOW_MSG_USER_START (TT_WINDOW_MSG_DEFAULT + 1)

// -- TtWindow -----------------------------------------------------------
class TtWindow {
public:
  static const int MSG_DEFAULT;

public:
  static void InitializeCommonControls( void );
  static LRESULT CALLBACK WindowProcedureForTTL( HWND handle, UINT msg, WPARAM w_param, LPARAM l_param );
  static TtWindow* FindWindow( HWND handle );

protected:
  static std::map<HWND, TtWindow*>& GetTtWindowTable( void );

private:
  static std::map<HWND, TtWindow*> TT_WINDOW_TABLE;

  // ---------------------------------------------------------------------
  // ȉ̃o֐͌pNXŃI[o[Ch邱
protected:
  virtual DWORD  GetStyle( void ) = 0;
  virtual DWORD  GetExtendedStyle( void ) = 0;
  virtual PCTSTR GetClassName( void ) = 0;
  virtual bool Created( void );

public:
  virtual LRESULT WindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT WMHorizontalScrollReceived( int code, int position, HWND handle );
  virtual LRESULT WMVerticalScrollReceived( int code, int position, HWND handle );
  virtual LRESULT DefaultWindowProcedure( UINT msg, WPARAM w_param, LPARAM l_param );

  explicit TtWindow( void );
  virtual ~TtWindow();

  // -- CreateParameter --------------------------------------------------
  struct CreateParameter {
  public:
    explicit CreateParameter( void );
    explicit CreateParameter( TtWindow* parent );
    explicit CreateParameter( TtWindow* parent, int id );
    explicit CreateParameter( const CreateParameter& other );

    CreateParameter AlterParent( TtWindow* parent ) const;
    CreateParameter AlterId( int id ) const;
    CreateParameter AlterInstanceHandle( HINSTANCE instance_handle ) const;
    CreateParameter AlterLPParam( LPVOID lp_param ) const;
    CreateParameter AlterIsChild( bool is_child ) const;
    CreateParameter AlterGroupStart( bool group_start ) const;

  public:
    TtWindow* parent_;
    int       id_;
    HINSTANCE instance_handle_;
    LPVOID    lp_param_;
    bool      is_child_;
    bool      group_start_;
  };

  void Create( void );
  void Create( CreateParameter parameter );
  virtual void CreatePrimitive( CreateParameter& parameter );

  int Close( void );

  int SendMessage( UINT msg, WPARAM w_param = 0, LPARAM l_param = 0 );
  void PostMessage( UINT msg, WPARAM w_param = 0, LPARAM l_param = 0 );
  LONG GetWindowLong( int index );
  void SetWindowLong( int index, LONG value );
  void OverrideWindowProcedure( void );

  TtWindow* GetParent( void );
  HWND      GetHandle( void );
  int       GetID( void );
  HINSTANCE GetInstanceHandle( void );

  RECT GetRectangle( void );
  int GetHorizontalPosition( void );
  int GetVerticalPosition( void );
  int GetWidth( void );
  int GetHeight( void );

  void Show( void );
  void Hide( void );
  void Minimize( void );
  void Maximize( void );
  void Restore( void );
  void SetShowState( int state );
  bool GetVisible( void );

  bool GetEnabled( void );
  void SetEnabled( bool flag );

  void SetActive( void );
  void SetFocus( void );

  void SetPosition( int x, int y );
  void SetSize( int width, int height );
  void SetPositionSize( int x, int y, int width, int height );
  void SetRectangle( RECT rect );
  void Move( int x, int y, int width, int height, int repaint );
  void SetCenterRelativeToParent( void );

  void SetBottom( void );
  void SetTop( void );
  void SetUnder( TtWindow& window );
  void SetUnder( HWND window );
  void SetTopMost( bool flag );

  void SetFont( TtFont& font );
  void SetIcon( HICON icon, bool is_big );

  int SetForground( void );

  int GetTextLength( void );
  std::string GetText( void );
  void SetText( const std::string& str );

protected:
  TtWindow* parent_;
  HWND      handle_;
  int       id_;
  HINSTANCE instance_handle_;
  WNDPROC   window_procedure_super_;
};

// -- TtExtraordinarilyWindow ----------------------------------------------------
class TtExtraordinarilyWindow : public TtWindow {
public:
  static TtExtraordinarilyWindow Desktop;

public:
  explicit TtExtraordinarilyWindow( HWND handle );
  explicit TtExtraordinarilyWindow( HWND handle, TtWindow& parent );

protected:
  virtual DWORD  GetStyle( void );
  virtual DWORD  GetExtendedStyle( void );
  virtual PCTSTR GetClassName( void );
};

#endif // TT_WINDOW_H_
